/**
* GMyth Library
*
* @file gmyth/gmyth_util.c
* 
* @brief <p> This component provides utility functions.
* 
* Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia.
* @author Hallyson Luiz de Morais Melo <hallyson.melo@indt.org.br>
*
*//*
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <glib.h>
#include <glib/gprintf.h>

#include "gmyth_util.h"

#include "gmyth_backendinfo.h"
#include "gmyth_socket.h"
#include "gmyth_programinfo.h"
#include "gmyth_common.h"
#include "gmyth_debug.h"


/** Converts a time_t struct in a GString at ISO standard format 
 * (e.g. 2006-07-20T09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the time value to be converted
 * @return GString* the converted isoformat string 
 */
GString*
gmyth_util_time_to_isoformat (time_t time_value)
{
	struct tm tm_time;
	GString *result;
	
	if (localtime_r(&time_value, &tm_time) == NULL) {
		g_warning ("gmyth_util_time_to_isoformat convertion error!\n");
        return NULL;
	}
	
	result = g_string_sized_new(20);
	g_string_printf(result, "%04d-%02d-%02dT%02d:%02d:%02d",
		tm_time.tm_year + 1900, tm_time.tm_mon + 1, tm_time.tm_mday,
		tm_time.tm_hour, tm_time.tm_min, tm_time.tm_sec);
	
	return result;
}

/** Converts a time_t struct in a GString to the following 
 * format (e.g. 2006-07-20 09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the time value to be converted
 * @return GString* the converted string 
 */
GString*
gmyth_util_time_to_string (time_t time_value)
{
	GString *result = gmyth_util_time_to_isoformat (time_value);
	result->str[10] = ' ';	

	return result;
}

/** Converts a GString in the following format 
 * (e.g. 2006-07-20 09:56:41) to a time_t struct.
 * 
 * @param time_str the string to be converted
 * @return time_t the time converted value
 */
time_t
gmyth_util_string_to_time (GString* time_str)
{
	int year, month, day, hour, min, sec;
    
#if 0
	gmyth_debug( "[%s] time_str = %s.\n", __FUNCTION__, time_str != NULL ? time_str->str : "[time string is NULL!]" );
#endif

	if (sscanf (time_str->str, "%04d-%02d-%02d %02d:%02d:%02d",
			&year, &month, &day, &hour, &min, &sec) < 3) { /* At least date */
		g_warning ("GMythUtil: isoformat_to_time converter error!\n");
		return 0;
	} else {
		struct tm tm_time;
		tm_time.tm_year = year - 1900;
		tm_time.tm_mon = month - 1;
		tm_time.tm_mday = day;
		tm_time.tm_hour = hour;
		tm_time.tm_min = min;
		tm_time.tm_sec = sec;
		
		return mktime (&tm_time);
	}
}

/** Decodes a long long variable from the string list
 * format of the myhtprotocol.
 * 
 * @param strlist the string list of mythprotocol values
 * @param offset  the list node offset of the long long variable
 * @return gint64  the long long converted value
 */
gint64
gmyth_util_decode_long_long(GMythStringList *strlist, guint offset) 
{

	gint64 ret_value = 0LL;

	g_return_val_if_fail( strlist != NULL, ret_value );

	if ( offset > gmyth_string_list_length( strlist ))
		g_printerr( "[%s] Offset is greater than the Stringlist (offset = %d)!\n", 
                    __FUNCTION__, offset );
    
	g_return_val_if_fail( offset < gmyth_string_list_length( strlist ), ret_value );

	gint l1 = gmyth_string_list_get_int( strlist, offset );
	gint l2 = gmyth_string_list_get_int( strlist, offset + 1 );

	ret_value = (l2 /*& 0xffffffffLL*/) | ( (gint64)l1 << 32 );

	return ret_value;

}

gboolean
gmyth_util_file_exists (GMythBackendInfo *backend_info, const char* filename)
{
    GMythSocket *socket;
    GMythProgramInfo *program;
    GMythStringList *slist;
    gboolean res;
    
    socket = gmyth_socket_new ();
    gmyth_socket_connect_to_backend (socket, backend_info->hostname,
		    backend_info->port, TRUE);

    program = g_new0 (GMythProgramInfo, 1);
    program->pathname = g_string_new (filename);

    slist = gmyth_string_list_new ();
    gmyth_string_list_append_char_array (slist, "QUERY_CHECKFILE");

    gmyth_program_info_to_string_list (program, slist);

    gmyth_socket_sendreceive_stringlist (socket, slist);

    res = (gmyth_string_list_get_int (slist, 0) == 1);
    
    // fixme: we should do this in a program_info_free() function
    g_string_free (program->pathname, FALSE);
    g_free (program);

    g_object_unref (slist);

    return res;    
}

